@@ -1,8 +1,9 @@
 #ifndef DBERN_H_
 #define DBERN_H_
 
-#include "RScalarDist.h"
+#include <distribution/RScalarDist.h>
 
+namespace jags {
 namespace bugs {
 
 /**
@@ -34,6 +35,6 @@ public:
 	      std::vector<double const *> const &par2) const;
 };
 
-}
+}}
 
 #endif /* DBERN_H_ */
