@@ -1,24 +1,25 @@
 #ifndef FUNC_MIN_H_
 #define FUNC_MIN_H_
 
-#include <function/VectorFunction.h>
+#include <function/ScalarVectorFunction.h>
 
+namespace jags {
 namespace bugs {
 
     /**
      * @short Minimum element of a vector
      * @see Max
      */
-    class Min: public VectorFunction
+    class Min: public ScalarVectorFunction
     {
     public:
 	Min ();
-	void evaluate(double *x, std::vector <double const *> const &args,
-		      std::vector<unsigned int> const &lengths) const;
-	bool checkParameterLength (std::vector<unsigned int> const &len) const;
+	double scalarEval (std::vector <double const *> const &args,
+			   std::vector<unsigned int> const &lengths) const;
 	/** Returns true if all arguments are true */
 	bool isDiscreteValued(std::vector<bool> const &mask) const;
     };
-}
+
+}}
 
 #endif /* FUNC_MIN_H_ */
